package it.amattioli.encapsulate.browsers;

import static it.amattioli.encapsulate.browsers.CalendarBrowser.IntervalType.*;

import it.amattioli.common.properties.PropertyChangeSupport;
import it.amattioli.dominate.Entity;
import it.amattioli.dominate.Repository;
import it.amattioli.dominate.memory.CollectionRepository;
import it.amattioli.encapsulate.dates.ConventionalTimeInterval;
import it.amattioli.encapsulate.dates.GenericTimeInterval;
import it.amattioli.encapsulate.dates.TimeInterval;
import it.amattioli.encapsulate.dates.specifications.DateSpecification;

import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.List;

public class CalendarBrowserImpl<I extends Serializable, T extends Entity<I>> implements CalendarBrowser<I,T> {
	private static final GenericTimeInterval UNLIMITED = new GenericTimeInterval((Date)null, (Date)null);
	private PropertyChangeSupport pChange = new PropertyChangeSupport(this);
	private IntervalType intervalType = MONTH;
	private ConventionalTimeInterval interval = intervalType.fromDate(new Date());
	private Repository<I, T> repository;
	private String dateProperty;
	private TimeInterval limits = UNLIMITED;
	
	public CalendarBrowserImpl() {
		
	}
	
	public CalendarBrowserImpl(Repository<I, T> repository, String dateProperty) {
		this.repository = repository;
		this.dateProperty = dateProperty;
	}

	public void setIntervalType(IntervalType intervalType) {
		IntervalType old = this.intervalType;
		this.intervalType = intervalType;
		interval = intervalType.fromDate(interval.getInitTime());
		pChange.firePropertyChange("intervalType", old, intervalType);
		pChange.firePropertyChange("interval", old, intervalType);
	}
	
	public IntervalType getIntervalType() {
		return intervalType;
	}
	
	public List<IntervalType> getIntervalTypeValues() {
		return Arrays.asList(new IntervalType[] {
			DAY,
			WEEK,
			MONTH
		});
	}
	
	public TimeInterval getLimits() {
		return limits;
	}

	public void setLimits(TimeInterval limits) {
		if (limits == null) {
			this.limits = UNLIMITED;
		} else {
			this.limits = limits;
			if (!interval.overlaps(limits)) {
				interval = intervalType.fromDate(limits.getLow());
			}
		}
	}

	public boolean hasNextInterval() {
		return !interval.next().after(limits);
	}
	
	public void nextInterval() {
		if (hasNextInterval()) {
			ConventionalTimeInterval old = interval;
			interval = interval.next();
			pChange.firePropertyChange("interval", old, interval);
		}
	}
	
	public boolean hasPreviousInterval() {
		return !interval.previous().before(limits);
	}

	public void previousInterval() {
		if (hasPreviousInterval()) {
			ConventionalTimeInterval old = interval;
			interval = interval.previous();
			pChange.firePropertyChange("interval", old, interval);
		}
	}
	
	public void position(Date time) {
		ConventionalTimeInterval old = interval;
		interval = intervalType.fromDate(time);
		pChange.firePropertyChange("interval", old, interval);
	}

	public void position(TimeInterval i) {
		ConventionalTimeInterval old = interval;
		interval = intervalType.fromInterval(i);
		pChange.firePropertyChange("interval", old, interval);
	}
	
	public void startPosition() {
		if (limits.isLowBounded()) {
			ConventionalTimeInterval old = interval;
			interval = intervalType.fromDate(limits.getLow());
			pChange.firePropertyChange("interval", old, interval);
		}
	}

	public ConventionalTimeInterval getInterval() {
		return interval;
	}

	@Override
	public Collection<T> getList() {
		DateSpecification<T> spec = DateSpecification.newInstance(dateProperty);
		spec.setValue(getInterval());
		return repository.list(spec);
	}

	public CalendarBrowser<I,T> subBrowser(IntervalType type) {
		Repository<I, T> subRepository = new CollectionRepository<I, T>(getList());
		CalendarBrowser<I,T> result = new CalendarBrowserImpl<I, T>(subRepository, dateProperty);
		result.setIntervalType(type);
		result.setLimits(getInterval());
		result.startPosition();
		return result;
	}
	
	public boolean hasSubBrowser() {
		return intervalType.hasPrevious();
	}
	
	public CalendarBrowser<I,T> subBrowser() {
		if (hasSubBrowser()) {
			return subBrowser(intervalType.previous());
		} else {
			return subBrowser(intervalType);
		}
	}
	
	public void addPropertyChangeListener(PropertyChangeListener listener) {
        pChange.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        pChange.removePropertyChangeListener(listener);
    }
}
